#!/usr/bin/env python

"""
Theming common functionality.

Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from os import listdir, makedirs
from os.path import exists, isfile, join, split
from shutil import copy

class Theme:

    "A common theme abstraction."

    def __init__(self, output, linker, pagename):

        """
        Initialise the theme with the given 'output' context, 'linker' and
        'pagename'.
        """

        self.output = output
        self.linker = linker
        self.pagename = pagename

    def apply(self, text):

        "Apply this theme to the given 'text', returning a themed version."

        return text

    def get_resource_base(self):

        "Return the filesystem base of resources for instances of this class."

        return split(self.__class__.origin)[0]

    def get_resource(self, filename):

        "Return the complete path for the resource with the given 'filename'."

        base = self.get_resource_base()
        return join(base, filename)

    def install_resource(self, filename, target=None):

        """
        Install the resource with the given 'filename' into a location having
        the given 'target' name (or 'filename' if 'target' is omitted).
        """

        pathname = self.get_resource(filename)
        outpath = self.output.get_filename(target or filename)

        self.copy(pathname, outpath)

    def copy(self, pathname, outpath):

        "Copy 'pathname' to 'outpath'."

        if isfile(pathname):
            outdir = split(outpath)[0]
            if outdir and not exists(outdir):
                makedirs(outdir)
            copy(pathname, outpath)
        else:
            if not exists(outpath):
                makedirs(outpath)
            for filename in listdir(pathname):
                self.copy(join(pathname, filename), join(outpath, filename))

    def load_resource(self, filename):

        "Return the textual content of the resource with the given 'filename'."

        f = open(self.get_resource(filename))
        try:
            return f.read()
        finally:
            f.close()

# vim: tabstop=4 expandtab shiftwidth=4
